@@ -17,7 +17,7 @@ sub new {
     $self->dir($top->current_path($from));
     $self->idx($index);
 
-    bless($self, $class);
+    $self->_last_dir_scanned(undef);
 
     $from->dir($self->dir());
 
@@ -32,6 +32,7 @@ use warnings;
 use base 'File::Find::Object::Base';
 
 __PACKAGE__->mk_accessors(qw(
+    _current_idx
     _dir_stack
     item
     _targets
@@ -54,7 +55,7 @@ __PACKAGE__->mk_accessors(@{__PACKAGE__->_get_options_ids()});
 
 use Carp;
 
-our $VERSION = '0.0.5';
+our $VERSION = '0.0.6';
 
 sub new {
     my ($class, $options, @targets) = @_;
@@ -72,6 +73,9 @@ sub new {
     }
     $tree->_targets([ @targets ]);
     $tree->_target_index(-1);
+    $tree->_current_idx(-1);
+
+    $tree->_last_dir_scanned(undef);
 
     return $tree;
 }
@@ -85,7 +89,17 @@ sub DESTROY {
 sub _current
 {
     my $self = shift;
-    return $self->_dir_stack()->[-1] || $self;
+
+    my $dir_stack = $self->_dir_stack();
+
+    if ($self->_current_idx < 0)
+    {
+        return $self;
+    }
+    else
+    {
+        return $dir_stack->[$self->_current_idx];
+    }
 }
 
 sub next {
@@ -110,6 +124,13 @@ sub _father
 {
     my ($self, $current) = @_;
 
+    if (!defined($current))
+    {
+        require Data::Dumper;
+        print Data::Dumper->new([$self],['$self'])->Dump();
+        confess "Current is undef";
+    }
+
     if (!defined($current->idx()))
     {
         return undef;
@@ -128,7 +149,7 @@ sub _movenext_with_current
 {
     my $self = shift;
     if ($self->_current->_curr_file(
-            shift(@{$self->_father($self->_current)->_files()})
+            shift(@{$self->_father($self->_current)->_traverse_to()})
        ))
     {
         $self->_current->_action({});
@@ -196,12 +217,14 @@ sub become_default
     if ($self eq $current)
     {
         @{$self->_dir_stack()} = ();
+        $self->_current_idx(-1);
     }
     else
     {
         while (scalar(@{$self->_dir_stack()}) != $current->idx() + 1)
         {
             pop(@{$self->_dir_stack()});
+            $self->_current_idx($self->_current_idx()-1);
         }
     }
 
@@ -217,7 +240,7 @@ sub _process_current {
     $self->isdot($current) and return 0;
     $self->_filter_wrapper($current) or return 0;  
 
-    foreach my $action ($self->depth() ? qw/b a/ : qw/a b/)
+    foreach my $action ($self->depth() ? qw(b a) : qw(a b))
     {
         if ($current->_action->{$action}) {
             next;
@@ -231,19 +254,40 @@ sub _process_current {
         }
             
         if ($action eq 'b') {
-            $self->check_subdir($current) or next;
-            push @{$self->_dir_stack()}, 
-                File::Find::Object::PathComponent->new(
-                    $self,
-                    $current, 
-                    scalar(@{$self->_dir_stack()})
-                );
-            return 0;
+            my $status = $self->_recurse($current);
+            
+            if ($status eq "SKIP")
+            {
+                next;
+            }
+            else
+            {
+                $self->_current_idx($self->_current_idx()+1);
+                return $status;
+            }
         }
     }
     return 0;
 }
 
+sub _recurse
+{
+    my ($self, $current) = @_;
+
+    $self->check_subdir($current) or 
+        return "SKIP";
+
+
+    push @{$self->_dir_stack()}, 
+        File::Find::Object::PathComponent->new(
+            $self,
+            $current,
+            scalar(@{$self->_dir_stack()})
+        );
+
+    return 0;
+}
+
 sub isdot
 {
     my ($self, $current) = @_;
@@ -306,23 +350,89 @@ sub current_path {
     }
 
     my $p = $self->_father($current)->dir();
-    $p =~ s!/+$!!; #!
-    $p .= '/' . $current->_curr_file;
-
-    return $p;
+    
+    return File::Spec->catfile($p, $current->_curr_file);
 }
 
 sub open_dir {
     my ($self, $current) = @_;
-    opendir(my $handle, $current->dir()) or return undef;
-    $current->_files(
-        [ sort { $a cmp $b } File::Spec->no_upwards(readdir($handle)) ]
-    );
+
+    if (defined($current->_last_dir_scanned()) &&
+        ($current->_last_dir_scanned() eq $current->dir()
+       )
+    )
+    {
+        return $current->_open_dir_ret();
+    }
+
+    $current->_last_dir_scanned($current->dir());
+
+    my $handle;
+    if (!opendir($handle, $current->dir()))
+    {
+        return $current->_open_dir_ret(undef);
+    }
+    my @files = (sort { $a cmp $b } File::Spec->no_upwards(readdir($handle)));
     closedir($handle);
+
+    $current->_files(
+        [ @files ]
+    );
+    $current->_traverse_to(
+        [ @files ]
+    );
+
+    
     my @st = stat($current->dir());
     $current->inode($st[1]);
     $current->dev($st[0]);
-    return 1;
+
+
+    return $current->_open_dir_ret(1);
+}
+
+sub set_traverse_to
+{
+    my ($self, $children) = @_;
+
+    # Make sure we scan the current directory for sub-items first.
+    $self->get_current_node_files_list();
+
+    $self->_current->_traverse_to([@$children]);
+}
+
+sub get_traverse_to
+{
+    my $self = shift;
+
+    return [ @{$self->_current->_traverse_to()} ];
+}
+
+sub get_current_node_files_list
+{
+    my $self = shift;
+
+    # Remming out because it doesn't work.
+    # $self->_father($self->_current)->dir($self->_current->dir());
+
+    $self->_current->dir($self->current_path($self->_current()));
+
+    # open_dir can return undef if $self->_current is not a directory.
+    if ($self->open_dir($self->_current))
+    {
+        return [ @{$self->_current->_files()}];
+    }
+    else
+    {
+        return [];
+    }
+}
+
+sub prune
+{
+    my $self = shift;
+
+    return $self->set_traverse_to([]);
 }
 
 1;
@@ -405,9 +515,28 @@ the scan is completed.
 Returns the current filename found by the File::Find::Object object, i.e: the
 last value returned by next().
 
+=head2 $ff->set_traverse_to([@children])
+
+Sets the children to traverse to from the current node. Useful for pruning
+items to traverse.
+
+=head2 $ff->prune()
+
+Prunes the current directory. Equivalent to $ff->set_traverse_to([]).
+
+=head2 [@children] = $ff->get_traverse_to()
+
+Retrieves the children that will be traversed to.
+
+=head2 [@files] = $ff->get_current_node_files_list()
+
+Gets all the files that appear in the current directory. This value is
+constant for every node, and is useful to use as the basis of the argument
+for C<set_traverse_to()>.
+
 =head1 BUGS
 
-Currently works only on UNIX as it uses '/' as a path separator.
+No bugs are known but it doesn't mean there aren't any.
 
 =head1 SEE ALSO
 
